/* newdex.c   Version 0.6 */

/* By Bill Kendrick */
/* (c) New Breed Software 1995 */

/* Started: 5/22/1995 */
/* Last change: 5/24/1995 */

/* A simple, recursive indexer for WWW home pages */
/* It will open an input .html file and list all links referenced
   in it with <a href> tags.
   Any local (without "http://") .html files which are referenced will
   be opened (if they haven't been opened before) and the same will be
   done for them.
   The output is a hierarchically-based (using nested unordered lists
   <ul>) .html file.
*/

/* Usage:    newdex [optionslist] inputfile output file    */
/* Options:  l - ok to add non-local references
             h - ok to add non-html references
             n - use filenames; don't try to find <title>'s
             r - raw output (not html format)
             v - verbose (show which local htmls are being searched through)
	     i - indents off
*/

#include "stdio.h"
#include "strings.h"

char seen[2048][200];   /* 2048 links.. too big a buffer?  NAH! */
int links,nonlocal,nonhtml,nameonly,raw,verbose,noindent;

char tolow(char c)
{
  if (c>='A' && c<='Z')
    c=c+32;
  return(c);
}

void findtitle(FILE * tf, char * dest)
{
  int c,done,len;
  char str[200];

  done=0;
  do
    {
      c=fgetc(tf);
      if (c=='<')
	{
	  len=0;
	  do
	    {
	      c=fgetc(tf);
	      if (c!='>' && c!=EOF)
		{
		  str[len]=tolow(c);
		  len++;
		}
	    }
	  while (c!='>' && c!=EOF);
	  str[len]='\0';
	  if (strcmp(str,"title")==0)
	    {
	      done=1;
	      len=0;
	      do
		{
		  c=fgetc(tf);
		  if (c!='<' && c!=EOF)
		    {
		      dest[len]=c;
		      len++;
		    }
		}
	      while (c!='<' && c!=EOF);
	      dest[len]='\0';
	    }
	}
    }
  while (c!=EOF && done==0);
}

void newdex(char path[], char in[], FILE * outfile, int deep)
{
  FILE * infile, * titlefile;
  int a,b,c,l,ll,any,ok;
  char s[500],ss[500],rin[500],pathf[500],inf[500],arg1[500],rpath[500];
  char callit[120],titlef[500];

  if (verbose==1)
    for (a=0; a<=deep; a++)
      printf("  ");

  any=0;

  strcpy(rin,path);       /* cat path and in name together for full name */
  strcat(rin,in);
  infile=fopen(rin,"r");

  if (infile!=NULL)
    {
      if (verbose==1)
	printf("Searching: %s\n",rin);
      do
	{
	  c=fgetc(infile);    /* read chars ... */
	  if (c=='<')         /* tag!? */
	    {
	      l=0;
	      do
	        {
	          c=fgetc(infile);   /* read into a string until end of tag */
		  if (c!=NULL)
		    s[l]=c;
	          l++;
	        }
	      while (c!=EOF && c!='>');
	      l=l-2;
	      s[l]='\0';
	      for (a=0; a<=7; a++)
		s[a]=tolow(s[a]);
	      if (s[0]=='a' && s[1]==32 && s[2]=='h'
		  && s[3]=='r' && s[4]=='e'
		  && s[5]=='f' && s[6]=='=' && s[7]==34)
		{                       /* is tag "a href=" ?? */
		  ll=l-1;
		  if ((ll>=13 && s[ll-4]=='.' && toupper(s[ll-3])=='H'
		       && toupper(s[ll-2])=='T' && toupper(s[ll-1])=='M'
		       && toupper(s[ll])=='L')
		      || (ll>=12 && s[ll-3]=='.' && toupper(s[ll-2])=='H'
			  && toupper(s[ll-1])=='T' && toupper(s[ll])=='M'))
		    {                    /* file is .html or .htm ?? */
		      for (a=8; a<=l; a++)
			ss[a-8]=s[a];
		      if (strstr(ss,":")==0)   /* contain ":"? no=local file */
			{
			  strcpy(arg1,ss);
                          if (any==0 && raw==0) /* ok, now we start a list.. */
                            {
			      if (noindent==0)
				for (a=0; a<=deep; a++)
				  fprintf(outfile,"  ");
                              fprintf(outfile,"<ul>\n");
                              any=1;
                            }
			  strcpy(pathf,path); /* pathf[0]='\0'; */
			  a=-1;
			  for (b=0; arg1[b]!='\0'; b++)   /* search for file */
			    {
			      if (arg1[b]=='/')
				a=b;
			    }
			  if (a==-1)    /* no "/" = no path, it's all file */
			    {
			      pathf[0]='\0';
			      strcpy(inf,arg1);
			    }
			  else   /* else extract the two halves.. */
			    {
			      for (b=0; b<=a; b++)
				pathf[b]=arg1[b];
			      pathf[a+1]='\0';
			      for (b=a+1; arg1[b]!='\0'; b++)
				inf[b-a-1]=arg1[b];
			      inf[b-a-1]='\0';
			    }
			  if (noindent==0)
			    for (a=0; a<=deep; a++)
			      fprintf(outfile,"  ");
			  strcpy(titlef,path);
			  strcat(titlef,pathf);
			  strcat(titlef,inf);
			  if (nameonly==0)
			    {
			      titlefile=fopen(titlef,"r");
                              strcpy(callit,ss);
			      if (titlefile!=NULL)
			        findtitle(titlefile,callit);
			      fclose(titlefile);
			    }
			  else
			    strcpy(callit,ss);
			  if (raw==0)
			    fprintf(outfile,"<li><a href=%c%s%s%c>%s</a>\n",34,path,s+8,34,callit);
			  else
			    fprintf(outfile,"%s\n",callit);
			  links++;
			  strcpy(seen[links],s+8);  /* add to seen list */
			  strcpy(rpath,path);   /* current path = old + new */
                          strcat(rpath,pathf);
			  ok=1;
			  for (a=0; a<links; a++)  /* seen before? */
			    {
			      if (strcmp(seen[a],s+8)==0)
				ok=0;
			    }
			  if (ok==1)   /* no? then do inside! */
			    newdex(rpath,inf,outfile,deep+1); /* recurse */
			}
		      else /* not local */
			if (nonlocal==1)
			  {
			    if (any==0 && raw==0) /* ok,now we start a list..*/
			      {
				if (noindent==0)
				  for (a=0; a<=deep; a++)
				    fprintf(outfile,"  ");
				fprintf(outfile,"<ul>\n");
				any=1;
			      }
			    if (noindent==0)
			      for (a=0; a<=deep; a++)
				fprintf(outfile,"  ");
			    if (raw==0)
			      fprintf(outfile,"<li><%s%c>%s</a>\n",s,34,s+8);
			    else
			      fprintf(outfile,"%s\n",s);
			  }
                    }
		  else /* not html */
		    if (nonhtml==1)
		      {
			if (any==0 && raw==0)  /* ok, now we start a list.. */
			  {
			    if (noindent==0)
			      for (a=0; a<=deep; a++)
				fprintf(outfile,"  ");
			    fprintf(outfile,"<ul>\n");
			    any=1;
			  }
			if (noindent==0)
			  for (a=0; a<=deep; a++)
			    fprintf(outfile,"  ");
			if (raw==0)
			  fprintf(outfile,"<li><%s%c>%s</a>\n",s,34,s+8);
			else
			  fprintf(outfile,"%s\n",s+8);
		      }
		}
      	      else
		; /* not anchor */
	    }
	  else
	    ; /* not tag */
	}
      while (c!=EOF);
      if (any==1 && raw==0)
	{
	  if (noindent==0)
	    for (a=0; a<=deep; a++)
	      fprintf(outfile,"  ");
	  fprintf(outfile,"</ul>\n");
	}
    }
  else
    printf("Can't open input file: %s\n",in);
  fclose(infile);
}

void main(int argv, char * argc[])
{
  char arg1[500],pathf[500],inf[500];
  int a,b;
  FILE * outf;

  links=0;
  nonlocal=0;
  nonhtml=0;
  nameonly=0;
  raw=0;
  verbose=0;
  noindent=0;
  printf("newdex v.0.6\nby Bill Kendrick\n(c) NBS 1995\n\n");
  if (argv==3 || argv==4)   /* proper args? */
    {
      if (argv==4)
	{
	  if (strchr(argc[1],'l')!=NULL)
	    nonlocal=1;
	  if (strchr(argc[1],'h')!=NULL)
	    nonhtml=1;
	  if (strchr(argc[1],'n')!=NULL)
	    nameonly=1;
	  if (strchr(argc[1],'r')!=NULL)
	    raw=1;
	  if (strchr(argc[1],'v')!=NULL)
	    verbose=1;
	  if (strchr(argc[1],'i')!=NULL)
	    noindent=1;
	  strcpy(arg1,argc[2]);
	  outf=fopen(argc[3],"w");
	}
      else
	{
	  strcpy(arg1,argc[1]);
	  outf=fopen(argc[2],"w");
	}
      if (outf!=NULL)
	{
	  if (raw==0)
	    {
	      fprintf(outf,"<title>Index</title>\n<h1>Index:</h1>\n");
	      fprintf(outf,"<hr>\n");   /* start HTML file ... */
	    }
          pathf[0]='\0';   /* no current path, let's look for one.. */
          a=-1;
          for (b=0; arg1[b]!='\0'; b++)
            {
              if (arg1[b]=='/')
                a=b;
            }
	  if (a==-1)    /* no specific path (only current) */
	    {
	      pathf[0]='\0';
	      strcpy(inf,arg1);
	    }
	  else   /* extract two halves... */
	    {
	      for (b=0; b<=a; b++)
		pathf[b]=arg1[b];
	      pathf[a+1]='\0';
	      for (b=a+1; arg1[b]!='\0'; b++)
		inf[b-(a+1)]=arg1[b];
	      inf[b-a-1]='\0';
	    }
	  strcpy(seen[0],inf);
          fprintf(outf,"%s:\n",seen[0]);
	  newdex(pathf,inf,outf,0);
          fclose(outf);
        }
      else
	printf("Can't open output file!\n");
    }
  else
    {
      printf("usage: newdex [hilnrv] input.html output.html\n");
      printf("  h = non-html references ok\n");
      printf("  i = indents off\n");
      printf("  l = non-local references ok\n");
      printf("  n = filenames only\n");
      printf("  r = raw output\n");
      printf("  v = verbose search listing\n");
    }
}
